/*
 **************************************************************************
 * Licensed Materials - Property of IBM
 * 5724-M24
 * Copyright IBM Corporation 2005,2012.  All rights reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 **************************************************************************
 * TextWrap.js
 *
 * Text wrapping
 */

/**
 * This class is used to do text wrapping
 *
 * The static '_init' method is called to find text wraps in the document and 
 * add the appropriately wrapped text.
 *
 * The class extracts information out of the SVG XML and calls the code in
 * AbtsractTextWrap.js to do the actual work.
 *
 * @param node - the XML/SVG node describing the text
 */ 
function TextWrap( node ) {
	this._node = node;
	this._node.normalize();

	// Get the parameters from the XML
	this._svg = SVG.addSVGSibling( node, document.createElementNS( SVG.ns, "text" ));
	this._svg.setAttribute( "class", "text" );
	this._svg.appendChild( document.createTextNode( "" ));
	this._svg.setAttribute( "style", this._node.getAttribute( "style" ));

	function onEmpty( x, def ) {
		return x == null || x == "" ? def : x;
	}

	this._x = +(onEmpty( node.getAttribute( "x" ), 0 ));
	this._y = +(onEmpty( node.getAttribute( "y" ), 0 ));
	var wordSep = onEmpty( node.getAttribute( "word-separator" ), null );
	if( wordSep != null ) wordSep = new RexExp( wordSep );
	AbstractTextWrap.call( this,
		this._node.firstChild != null ? this._node.firstChild.data : "",
		onEmpty( node.getAttribute( "width" ), null ),
		onEmpty( node.getAttribute( "height" ), null ),
		onEmpty( node.getAttribute( "font-size" ), null ),
		onEmpty( node.getAttribute( "line-interval" ), null ),
		onEmpty( node.getAttribute( "horizontal-fill" ), null ),
		onEmpty( node.getAttribute( "vertical-fill" ), null ),
		onEmpty( node.getAttribute( "horizontal-align" ), null ),
		onEmpty( node.getAttribute( "vertical-align" ), null ),
		wordSep,
		onEmpty( node.getAttribute( "hyphen" ), null ),
		onEmpty( node.getAttribute( "continuation" ), null )
	);

	if (node.parentNode) {
		this._x = SVG.convertLengthToUserSpace( this._x, node.parentNode);
		this._y = SVG.convertLengthToUserSpace( this._y, node.parentNode);
		this._width = SVG.convertLengthToUserSpace( this._width, node.parentNode);
		this._height = SVG.convertLengthToUserSpace( this._height, node.parentNode);
		this._fontSize = SVG.convertLengthToUserSpace( this._fontSize, node.parentNode);
	}
	this._svg.setAttribute( "font-size", this._fontSize);
	this._lineInterval = SVG.convertLengthToUserSpace( this._lineInterval, this._svg );

	this.wrap();
}

TextWrap.prototype = new AbstractTextWrap();
TextWrap.prototype._x = 0;
TextWrap.prototype._y = 0;
TextWrap.prototype._lineInterval = "1em";

/**
 * Override getTextWidth in AbstractTextWrap.
 * 
 * Delegates to the SVG function, getComputedTextLength.
 */
TextWrap.prototype.getTextWidth = function( string, fontSize ) {
	if( string.length == 0 ) return 0;
	this._svg.setAttribute( "font-size", fontSize);
	this._svg.firstChild.data = string
	var width = this._svg.getComputedTextLength();	
	this._svg.firstChild.data = "";
	return width;
}

/**
 * Override outputLine in AbstractTextWrap.
 *
 * Creates a tspan to contain the line.
 * May also create an textPath if the text should lie on some path.
 */
TextWrap.prototype.outputLine = function( line, x, y, fontSize, wordSpacing ) {
	var tspan = document.createElementNS( SVG.ns, "tspan" );
	tspan.appendChild( document.createTextNode( line )); 
	if( wordSpacing != null ) {
		tspan.style.setProperty( "word-spacing", wordSpacing, "important" );
	}
	tspan.setAttribute( "x", x + this._x );
	tspan.setAttribute( "y", y + this._y );
	tspan.setAttribute( "font-size", fontSize);
	tspan.setAttribute( "dy", "1em" );
	
	this._svg.appendChild( tspan );
}

/**
 * Namespace of text wrap elements
 */
TextWrap.ns = "http://www.lombardi.com/bpd/svg/";
/**
 * The tag name for this node
 */
TextWrap.tag = "text-wrap";

/**
 * A list of instances that are found when '_init' is called
 */
TextWrap._instances = new Array();

/**
 * Find all the text-wrap elements and wrap them.
 */
TextWrap._init = function () {
	var elements = document.documentElement.getElementsByTagNameNS( this.ns, this.tag );
	for( var i = 0; i < elements.length; i++ ) {
		this._instances.push( new TextWrap( elements.item( i )));
	}
}
